/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef EARL_SEQTRACE_H
#define EARL_SEQTRACE_H

/*
 *----------------------------------------------------------------------------
 *
 * class SeqTrace
 *
 * Trace with sequential access to events
 *
 *----------------------------------------------------------------------------
 */

#include <cstdio>
#include <string>
#include <map>

#include "Callsite.h"
#include "Event.h"
#include "EventBuffer.h"
#include "Grid.h"
#include "Metric.h"
#include "Region.h"
#include "State.h"

typedef struct ElgIn_struct ElgIn;

namespace earl
{
  class SeqTrace
  {
  public:

    SeqTrace(std::string path);
    ~SeqTrace();

    long       get_nevents()  const { return nevents; }

    long       get_nfiles()   const { return filem.size(); }
    long       get_nregs()    const { return regm.size(); }
    long       get_ncsites()  const { return csitem.size(); }
    long       get_nmets()    const { return metm.size(); }

    Grid*      get_grid()     const { return grid; }

    std::string   get_file(long file_id)   const;
    Region*       get_reg(long reg_id)     const;
    Callsite*     get_csite(long csite_id) const;

    Metric*       get_met(long met_id) const;

  protected:
  
    Event    next_event(State& state, EventBuffer* buffer);
    void     set_offset(const fpos_t* offset);
    fpos_t   get_offset();

  private:
  
    bool skip_event();

    std::string   get_str(long strid)    const;

    std::map<long, std::string>    strm;
    std::map<long, std::string>    filem;
    std::map<long, Region*>        regm;
    std::map<long, Callsite*>      csitem;
    std::map<long, Metric*>        metm;

    Region* dump;
    Region* logoff;

    Grid* grid;
    long nevents;

    ElgIn* file;
  };
}
#endif























