%typemap(python, in) long 
{
  if (PyInt_Check($source))
    {
      $target = PyInt_AsLong($source);
    }
  else if (PyLong_Check($source))
    {
      $target = PyLong_AsLong($source);
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not an integer");
      return NULL;
    }
}

%typemap(python, in) string* (string str)
{
  $target = &str;	

  if (PyString_Check($source))
    {
      *$target = PyString_AsString($source);
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not a string");
      return NULL;
    }
}

%typemap(python, in) etype* (etype et)
{
  $target = &et;

  if (PyString_Check($source))
    {
      if ( strcmp(PyString_AsString($source), "ALOCK") == 0 )
	*$target = ALOCK;
      else if ( strcmp(PyString_AsString($source), "ENTER") == 0 )
	*$target = ENTER;
      else if ( strcmp(PyString_AsString($source), "EVENT") == 0 )
	*$target = EVENT;
      else if ( strcmp(PyString_AsString($source), "EXIT") == 0 )
	*$target = EXIT;
      else if ( strcmp(PyString_AsString($source), "FLOW") == 0 )
	*$target = FLOW;
      else if ( strcmp(PyString_AsString($source), "FORK") == 0 )
	*$target = FORK;
      else if ( strcmp(PyString_AsString($source), "JOIN") == 0 )
	*$target = JOIN;
      else if ( strcmp(PyString_AsString($source), "MPICEXIT") == 0 )
	*$target = MPICEXIT;
      else if ( strcmp(PyString_AsString($source), "OMPCEXIT") == 0 )
	*$target = OMPCEXIT;
      else if ( strcmp(PyString_AsString($source), "P2P") == 0 )
	*$target = P2P;
      else if ( strcmp(PyString_AsString($source), "RECV") == 0 )
	*$target = RECV;
      else if ( strcmp(PyString_AsString($source), "RLOCK") == 0 )
	*$target = RLOCK;
      else if ( strcmp(PyString_AsString($source), "SEND") == 0 )
	*$target = SEND;
      else if ( strcmp(PyString_AsString($source), "SYNC") == 0 )
	*$target = SYNC;
      else if ( strcmp(PyString_AsString($source), "TEAM") == 0 )
	*$target = TEAM;
      else 
	{
	  PyErr_SetString(PyExc_TypeError, "no such type");
	  return NULL;
	}
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not a string");
      return NULL;
    }
}

%typemap(python, out) string 
{
  $target = PyString_FromString(const_cast<char *>($source->c_str()));
}

%typemap(python, ret) string 
{
  delete $source;
}


%typemap(python, out) Location*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_id());  
  PyDict_SetItemString($target, "id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_mach()->get_id());  
  PyDict_SetItemString($target, "mach_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_node()->get_node_id());  
  PyDict_SetItemString($target, "node_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_proc()->get_id());  
  PyDict_SetItemString($target, "proc_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_thrd()->get_thrd_id());  
  PyDict_SetItemString($target, "thrd_id", item);
  Py_DECREF(item);
}

%typemap(python, out) Machine*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_id());  
  PyDict_SetItemString($target, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_name().c_str()));   
  PyDict_SetItemString($target, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_nnodes());  
  PyDict_SetItemString($target, "nnodes", item);
  Py_DECREF(item);
}

%typemap(python, out) Node*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_mach_id());  
  PyDict_SetItemString($target, "mach_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_node_id());  
  PyDict_SetItemString($target, "node_id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_name().c_str()));   
  PyDict_SetItemString($target, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_ncpus());  
  PyDict_SetItemString($target, "ncpus", item);
  Py_DECREF(item);
  item = PyFloat_FromDouble($source->get_clckrt());  
  PyDict_SetItemString($target, "clckrt", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_nprocs());  
  PyDict_SetItemString($target, "nprocs", item);
  Py_DECREF(item);
}

%typemap(python, out) Process*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_id());  
  PyDict_SetItemString($target, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_name().c_str()));   
  PyDict_SetItemString($target, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_nthrds());  
  PyDict_SetItemString($target, "nthrds", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_loc()->get_id());  
  PyDict_SetItemString($target, "loc_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_node()->get_node_id());  
  PyDict_SetItemString($target, "node_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_node()->get_mach_id());  
  PyDict_SetItemString($target, "mach_id", item);
  Py_DECREF(item);
}

%typemap(python, out) Thread*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_proc_id());  
  PyDict_SetItemString($target, "proc_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_thrd_id());  
  PyDict_SetItemString($target, "thrd_id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_name().c_str()));   
  PyDict_SetItemString($target, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_loc()->get_id());  
  PyDict_SetItemString($target, "loc_id", item);
  Py_DECREF(item);
}

%typemap(python, out) Callsite*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_id());  
  PyDict_SetItemString($target, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_file().c_str()));   
  PyDict_SetItemString($target, "file", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_line());  
  PyDict_SetItemString($target, "line", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_callee()->get_id());   
  PyDict_SetItemString($target, "callee_id", item);
  Py_DECREF(item);
}


%typemap(python, out) Region*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_id());  
  PyDict_SetItemString($target, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_name().c_str()));   
  PyDict_SetItemString($target, "name", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_file().c_str()));   
  PyDict_SetItemString($target, "file", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_begln());  
  PyDict_SetItemString($target, "begln", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_endln());  
  PyDict_SetItemString($target, "endln", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_rtype().c_str()));   
  PyDict_SetItemString($target, "rtype", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_descr().c_str()));   
  PyDict_SetItemString($target, "descr", item);
  Py_DECREF(item);
}

%typemap(python, out) Metric*
{
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_id());  
  PyDict_SetItemString($target, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_name().c_str()));   
  PyDict_SetItemString($target, "name", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_descr().c_str()));   
  PyDict_SetItemString($target, "descr", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_type().c_str()));   
  PyDict_SetItemString($target, "type", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_mode().c_str()));   
  PyDict_SetItemString($target, "mode", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_ival().c_str()));   
  PyDict_SetItemString($target, "ival", item);
  Py_DECREF(item);
}

%typemap(python, out) Communicator*
{ 
  $target = PyList_New($source->size());
  
  for ( int i = 0; i < $source->size(); i++)
    {
      PyObject* item = PyInt_FromLong($source->get_proc(i)->get_id());
      PyList_SetItem($target, i, item);
    }
}

%typemap(python, out) Event
{ 
  PyObject* item;

  $target = PyDict_New();

  item = PyInt_FromLong($source->get_pos());  
  PyDict_SetItemString($target, "pos", item);
  Py_DECREF(item);
  item = PyInt_FromLong($source->get_loc()->get_id());  
  PyDict_SetItemString($target, "loc_id", item);
  Py_DECREF(item);
  item = PyFloat_FromDouble($source->get_time());  
  PyDict_SetItemString($target, "time", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($source->get_typestr().c_str()));   
  PyDict_SetItemString($target, "type", item);
  Py_DECREF(item);
  if ( ! $source->get_enterptr().null() )
    item = PyInt_FromLong($source->get_enterptr().get_pos());  
  else
    item = PyInt_FromLong(-1);
  PyDict_SetItemString($target, "enterptr", item);
  Py_DECREF(item);

  if ( $source->is_type(FLOW) )
    {
      item = PyInt_FromLong($source->get_reg()->get_id());  
      PyDict_SetItemString($target, "reg_id", item);
      Py_DECREF(item);

      for ( int i = 0; i < $source->get_nmets(); i++ )	
	{
	  item = PyLong_FromLongLong($source->get_metval(i));  
	  PyDict_SetItemString($target, const_cast<char*>($source->get_metname(i).c_str()), item);
	  Py_DECREF(item);
	}
    }
  if ( $source->is_type(ENTER) )
    {
      item = PyInt_FromLong($source->get_cnodeptr().get_pos());  
      PyDict_SetItemString($target, "cnodeptr", item);
      Py_DECREF(item);
      if ( ! $source->get_cedgeptr().null() )
	item = PyInt_FromLong($source->get_cedgeptr().get_pos());  
      else
	item = PyInt_FromLong(-1);
      PyDict_SetItemString($target, "cedgeptr", item);
      Py_DECREF(item);

      if ( $source->get_csite() )
	item = PyInt_FromLong($source->get_csite()->get_id() );  
      else
	item = PyInt_FromLong(-1);
      PyDict_SetItemString($target, "csite_id", item);
      Py_DECREF(item);
    }
  if ( $source->is_type(MPICEXIT) )
    {
      if ( $source->get_root() != NULL )
	item = PyInt_FromLong($source->get_root()->get_id());
      else
	item = PyInt_FromLong(-1);	
      PyDict_SetItemString($target, "root_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($source->get_com()->get_id());  
      PyDict_SetItemString($target, "com_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($source->get_sent());  
      PyDict_SetItemString($target, "sent", item);
      Py_DECREF(item);
      item = PyInt_FromLong($source->get_recvd());  
      PyDict_SetItemString($target, "recvd", item);
      Py_DECREF(item);
    }
  if ( $source->is_type(P2P) )
    {
      item = PyInt_FromLong($source->get_com()->get_id());  
      PyDict_SetItemString($target, "com_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($source->get_tag());  
      PyDict_SetItemString($target, "tag", item);
      Py_DECREF(item);
      item = PyInt_FromLong($source->get_length());  
      PyDict_SetItemString($target, "length", item);
      Py_DECREF(item);
    }
  if ( $source->is_type(SEND) )
    {
      item = PyInt_FromLong($source->get_dest()->get_id());  
      PyDict_SetItemString($target, "dest_id", item);
      Py_DECREF(item);
    }     
  if ( $source->is_type(RECV) )
    {
      item = PyInt_FromLong($source->get_src()->get_id());  
      PyDict_SetItemString($target, "src_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($source->get_sendptr().get_pos());  
      PyDict_SetItemString($target, "sendptr", item);
      Py_DECREF(item);
    }     
  if ( $source->is_type(SYNC) )
    {
      item = PyInt_FromLong($source->get_lock_id());  
      PyDict_SetItemString($target, "lock_id", item);
      Py_DECREF(item);
      if ( ! $source->get_lockptr().null() )
	item = PyInt_FromLong($source->get_lockptr().get_pos());  
      else
	item = PyInt_FromLong(-1);
      PyDict_SetItemString($target, "lockptr", item);
      Py_DECREF(item);
    }
  if ( $source->is_type(JOIN) )
    {
      item = PyInt_FromLong($source->get_forkptr().get_pos());  
      PyDict_SetItemString($target, "forkptr", item);
      Py_DECREF(item);
    }
}

%typemap(python, ret) Event
{
  delete $source;
}

%typemap(python, ignore) Eventv& (Eventv ev)
{
  $target = &ev;
}

%typemap(python, argout) Eventv&
{ 
  $target = PyList_New($source->size());
  
  for ( int i = 0; i < $source->size(); i++)
    {
      PyObject* item = PyInt_FromLong((*$source)[i].get_pos());
      PyList_SetItem($target, i, item);
    }
}

%except(python) 
{	
  try 	
    {	
      $function
    }	
  catch(RuntimeError e) 
    {
      PyErr_SetString(PyExc_RuntimeError, const_cast<char*>(e.get_msg().c_str()));   
      return NULL;
    }
  catch(FatalError e) 
    {
      PyErr_SetString(PyExc_SystemExit, const_cast<char*>(e.get_msg().c_str()));   
      return NULL;
    }
}













