/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                ** 
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         ** 
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "dlg_csite.h"
#include "elg_error.h"

#include <string.h>
#include <stdlib.h>

struct dlg_csite
{
  long id;
  long mod_id;
  long line;
  long caller_id;
  long callee_id;
  long tag;
};

struct dlg_csite*   
dlg_csite_create(char* csitespec)
{
  struct dlg_csite* csite = malloc(sizeof(struct dlg_csite));
  if ( csite == NULL )
    elg_error();

  if ( strcmp(strtok(csitespec, " "), "CALLSITE") != 0 )
    elg_error_msg("Invalid call-site specification string");
  
  csite->id        = atol(strtok(NULL, " "));
  csite->mod_id    = atol(strtok(NULL, " "));
  csite->line      = atol(strtok(NULL, " "));
  csite->caller_id = atol(strtok(NULL, " "));
  csite->callee_id = atol(strtok(NULL, " "));
  
  csite->tag = -1;

  return csite;
}

void              
dlg_csite_delete(struct dlg_csite* csite)
{
  free(csite);
}

long              
dlg_csite_id(struct dlg_csite* csite)
{
  return csite->id;
}

long              
dlg_csite_mod_id(struct dlg_csite* csite)
{
  return csite->mod_id;
}

long              
dlg_csite_line(struct dlg_csite* csite)
{
  return csite->line;
}

long              
dlg_csite_caller_id(struct dlg_csite* csite)
{
  return csite->caller_id;
}

long              
dlg_csite_callee_id(struct dlg_csite* csite)
{
  return csite->callee_id;
}

long              
dlg_csite_get_tag(struct dlg_csite* csite)
{
  return csite->tag;
}

void              
dlg_csite_set_tag(struct dlg_csite* csite, long tag)
{
  csite->tag = tag;
}

void             
dlg_csite_fprintf(FILE* stream, struct dlg_csite* csite)
{
  int i;
  
  fprintf(stream, 
	  "CALLSITE %d %d %d %d %d %d", 
	  csite->id, 
	  csite->mod_id, 
	  csite->line, 
	  csite->caller_id, 
	  csite->callee_id);  
  fprintf(stream, " tag = %d\n", csite->tag);
}
