/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                ** 
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         ** 
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "dlg_func.h"
#include "elg_error.h"

#include <string.h>
#include <stdlib.h>

#include <stdio.h>

struct dlg_func
{
  long id;
  char* name;
  char* descr;
  long begln;
  long endln;
  long mod_id;
  long ncsites;
  long* csite_idv;
  long tag;
};

struct dlg_func*   
dlg_func_create(char* funcspec)
{
  char* name;
  int i;
  char* tmp;
  
  struct dlg_func* func = malloc(sizeof(struct dlg_func));
  if ( func == NULL )
    elg_error();

  if ( strcmp(strtok(funcspec, " "), "FUNCTION") != 0 )
    elg_error_msg("Invalid function-specification string");
  
  func->id = atol(strtok(NULL, " "));

  name = strtok(NULL, " ");
  func->name = malloc(strlen(name) + 1);
  if ( func->name == NULL )
    elg_error();
  memcpy(func->name, name, strlen(name) + 1);
  
  func->begln   = atol(strtok(NULL, " "));
  func->endln   = atol(strtok(NULL, " "));
  func->mod_id  = atol(strtok(NULL, " "));
  func->ncsites = atol(strtok(NULL, " "));
  
  if ( func->ncsites > 0 )
    {
      func->csite_idv = calloc(func->ncsites, sizeof(long));
      if ( func->csite_idv == NULL )
	elg_error();
    }
  else
    {
      func->csite_idv = NULL;
    }
  
  for ( i = 0; i < func->ncsites; i++)
    func->csite_idv[i] = atol(strtok(NULL, " "));

  func->tag = -1;

  /* description */
  if ( strncmp(func->name, "MPI", 3) == 0 || strncmp(func->name, "mpi", 3) == 0 )
    func->descr = "MPI";
  else if ( strncmp(func->name, "OMP", 3) == 0 || strncmp(func->name, "omp", 3) == 0 )
    func->descr = "OpenMP";
  else
    func->descr = "USR";

  return func;
}

void              
dlg_func_delete(struct dlg_func* func)
{
  free(func->name);
  free(func->csite_idv);
  free(func);
}

long              
dlg_func_id(struct dlg_func* func)
{
  return func->id;
}

const char*      
dlg_func_name(struct dlg_func* func)
{
  return func->name;
}

long              
dlg_func_begln(struct dlg_func* func)
{
  return func->begln;
}

long              
dlg_func_endln(struct dlg_func* func)
{
  return func->endln;
}

long              
dlg_func_mod_id(struct dlg_func* func)
{
  return func->mod_id;
}

long              
dlg_func_num_funcs(struct dlg_func* func)
{
  return func->ncsites;
}

long  
dlg_func_csite_id(struct dlg_func* func, long i)
{
  return func->csite_idv[i];
}

long              
dlg_func_get_tag(struct dlg_func* func)
{
  return func->tag;
}

void              
dlg_func_set_tag(struct dlg_func* func, long tag)
{
  func->tag = tag;
}

void             
dlg_func_fprintf(FILE* stream, struct dlg_func* func)
{
  int i;
  
  fprintf(stream, 
	  "FUNCTION %d %s %s %d %d %d %d", 
	  func->id, 
	  func->name, 
	  func->descr, 
	  func->begln, 
	  func->endln, 
	  func->mod_id, 
	  func->ncsites);  
  for ( i = 0; i < func->ncsites; i++ )
    fprintf(stream, " %d", func->csite_idv[i]);
  fprintf(stream, " tag = %d\n", func->tag);
}

const char*       
dlg_func_descr(struct dlg_func* func)
{
  return func->descr;
}
