/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_comp_pgi.h"
#include "elg_defs.h"
#include "elg_env.h"
#include "elg_mrgproc.h"
#include "elg_trc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#if (defined (ELG_OMPI) || defined (ELG_OMP))
#define ELG_MY_THREAD   omp_get_thread_num() 
#define ELG_NUM_THREADS omp_get_max_threads() 
#else
#define ELG_MY_THREAD   0
#define ELG_NUM_THREADS 1 
#endif

long* cstsizev;

extern void decode_identifier(char *id, char *output_buffer,
                              size_t  output_buffer_size,
                              int *err, int *buffer_overflow_err,
                              size_t  *required_buffer_size);



/*
 *-----------------------------------------------------------------------------
 * called at the beginning of each profiled routine
 *----------------------------------------------------------------------------- */

void ___rouent2(struct s1 *p) {

  if (!p->isseen)
    {
      static char dbuf[512];
      char* rname; 
      int err, err_buf;
      size_t rbs;      
      elg_ui4 fid, fnid, rid, rnid, rdid; 
      char* modpos;
      
      /* demangle name */
      decode_identifier(p->rout, dbuf, sizeof(dbuf), &err, &err_buf, &rbs);
      if (!err)
	rname =  p->rout;
      else
	elg_error_msg("Can't decode PGI function name.");

      /* fix opari output file names */
      if ( (modpos = strstr(p->file, ".mod.")) != NULL )
        {
          strcpy(modpos, modpos+4);
        }
      
#if (defined (ELG_OMPI)  || defined (ELG_OMP))

      if (omp_in_parallel())
	{
#pragma omp critical (elg_comp_pgi_1)
	  {
	    if (!p->isseen)
	      {	
		p->fid    = elg_def_file(p->file);
		p->rid    = elg_def_region(rname, p->fid, p->lineno, ELG_NO_LNO, "USR", ELG_FUNCTION);
		p->isseen = 1;
	      }
	  }
	}
      else
	{
	  p->fid    = elg_def_file(p->file);
	  p->rid    = elg_def_region(rname, p->fid, p->lineno, ELG_NO_LNO, "USR", ELG_FUNCTION);
	  p->isseen = 1;
	}
#else
 
      p->fid    = elg_def_file(p->file);
      p->rid    = elg_def_region(rname, p->fid, p->lineno, ELG_NO_LNO, "USR", ELG_FUNCTION);
      p->isseen = 1;

#endif
    }
      
  /* increment call-stack size */
  cstsizev[ELG_MY_THREAD]++; 
  
  /* write enter trace record */
  elg_enter(p->rid);  
}

/*
 *-----------------------------------------------------------------------------
 * called at the end of each profiled routine
 *-----------------------------------------------------------------------------
 */

void ___rouret2(void) {
  
  /* decrement call-stack size */
  cstsizev[ELG_MY_THREAD]--; 

  elg_exit();
}

void ___linent2(void *l) {
}

/*
 *-----------------------------------------------------------------------------
 * called during program termination
 *-----------------------------------------------------------------------------
 */

void __rouexit() {

  /* write pending exits */
  while(cstsizev[ELG_MY_THREAD] > 0)
    {
      cstsizev[ELG_MY_THREAD]--; 
      elg_exit();
    }

  /* free call-stack-size vector */
  free(cstsizev);

  /* close trace file */
  elg_close();
}


/*
 *-----------------------------------------------------------------------------
 * called during program initialization
 * - Note, linux does not pass the program name but garbage pointer
 *-----------------------------------------------------------------------------
 */

void __rouinit(char *pgm) {
 
  int i;

  /* call-stack initialization */
  cstsizev = calloc(ELG_NUM_THREADS, sizeof(long)); 
  for (i = 0; i < ELG_NUM_THREADS; i++)
    cstsizev[i] = 0;
 
  /* open trace file */
  elg_open();

  atexit(__rouexit);
}
















