#include <stdlib.h>
#include <string.h>
#include "elg_trc.h"

/*
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - SUN f90 compiler PHAT interface
 * 
 *-----------------------------------------------------------------------------
 */

/*
 *-----------------------------------------------------------------------------
 * Simple hash table to map function names to region identifier
 *-----------------------------------------------------------------------------
 */

typedef struct HN {
  long id;            /* hash code (address of function name) */
  elg_ui4 elgid;      /* associated EPILOG region identifier  */
  struct HN* next;
} HashNode;

#define HASH_MAX 1024

static HashNode* htab[HASH_MAX];

/*
 * Stores EPILOG region identifier `e' under hash code `h'
 */

static void hash_put(long h, elg_ui4 e) {
  long id = h % HASH_MAX;
  HashNode *add = malloc(sizeof(HashNode));
  add->id = h;
  add->elgid = e;
  add->next = htab[id];
  htab[id] = add;
}

/*
 * Lookup hash code `h'
 * Returns EPILOG region identifier if already stored, otherwise ELG_NO_ID
 */

static elg_ui4 hash_get(long h) {
  long id = h % HASH_MAX;
  HashNode *curr = htab[id];
  while ( curr ) {
    if ( curr->id == h ) {
      return curr->elgid;
    }
    curr = curr->next;
  }
  return ELG_NO_ID;
}

/*
 * Register new region with EPILOG system
 * `str' is passed in from SUN compiler
 */

static elg_ui4 register_region(char *str) {
  elg_ui4 rid;
  elg_ui4 fid;

  /* -- register region with EPILOG and store region identifier -- */
  rid = elg_def_region(str, ELG_NO_ID, ELG_NO_LNO, ELG_NO_LNO, "USR", ELG_FUNCTION);
  hash_put((long) str, rid);
  return rid;
}

/*
 * This function is called at program end
 */

void phat_elg_finalize () { 
  elg_exit(); /*main*/
  elg_close();
}

/*
 * This function is called at program start
 */

#pragma init (phat_elg_init)
void phat_elg_init () { 
  elg_ui4 rid;

  elg_open();
  rid = register_region("main");
  elg_enter(rid);
  atexit(phat_elg_finalize);
}

/*
 * This function is called at the entry of each function
 * The call is generated by the SUN f90 compilers
 */

void phat_enter(char *str, int *id) {
  /* -- ignore SUN OMP runtime functions -- */
  if ( strchr(str, '$') != NULL ) return;

  /* -- get region identifier -- */
  if ( *id == -1 ) {
    /* -- region entered the first time, register region -- */
#   if defined (ELG_OMPI) || defined (ELG_OMP)
    if (omp_in_parallel()) {
#     pragma omp critical (elg_comp_phat_1)
      {
        if ( (*id = hash_get((long) str)) == ELG_NO_ID ) {
          *id = register_region(str);
        }
      }
    } else {
      *id = register_region(str);
    }
#   else
    *id = register_region(str);
#   endif
  }

  /* -- write EPILOG enter record -- */
  elg_enter(*id);
}


/*
 * This function is called at the exit of each function
 * The call is generated by the SUN F90 compilers
 */

void phat_exit(char *str, int *id) {
  if ( *id == -1 ) return;

  /* -- write EPILOG exit record -- */
  elg_exit();
}
