/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_error.h"

#include "elg_env.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define ELG_MSG_PFIX  "EPILOG"
#define ELG_MSG_SIZE  4096

static void elg_print_msg(const char* fmt, va_list az)
{
  char buffer[ELG_MSG_SIZE];
  
  sprintf(buffer, "%s: ", ELG_MSG_PFIX);
  vsprintf(buffer + strlen(buffer), fmt, az);
  fprintf(stderr, "%s\n", buffer);
  fflush(NULL);
  return;
}

void elg_error()
{
  perror(ELG_MSG_PFIX);
  fflush(stderr);
  exit(EXIT_FAILURE);
}

void elg_error_msg(const char* fmt, ...)
{
  va_list ap;

  va_start(ap, fmt);
  elg_print_msg(fmt, ap);
  exit(EXIT_FAILURE);
}

void elg_warning(const char* fmt, ...)
{
  va_list ap;

  va_start(ap, fmt);
  elg_print_msg(fmt, ap);
  return;
}

void elg_cntl_msg(const char* fmt, ...)
{
  va_list ap;

  if (elg_env_is_verbose())
    {
      va_start(ap, fmt);
      elg_print_msg(fmt, ap);
      return;
    }
}






