/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_pform.h"
#include "elg_defs.h"
#include "elg_error.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>

#ifndef ELG_PROCDIR 
#  define ELG_PROCDIR "/proc/"
#endif

static elg_ui8       elg_cycles_per_sec=1;
static unsigned char elg_cpu_has_tsc=0;
static unsigned int  elg_cpu_count=0;

void elg_pform_init()
{
  FILE   *cpuinfofp;
  char   line[1024];
  char   *token;
  
  /* size corresponding to elg_cycles_per_sec */
  elg_d8 hz;
  
  if ((cpuinfofp = fopen (ELG_PROCDIR "cpuinfo", "r")) == NULL) 
    elg_error_msg("Cannot open file %s: %s\n", ELG_PROCDIR "cpuinfo", strerror(errno));
  
  while (fgets(line, sizeof (line), cpuinfofp))
  {
    if (!strncmp("processor", line, 9))
      elg_cpu_count++;
    if (!strncmp("cpu MHz", line, 7))
    {
      strtok(line, ":");
      
      hz = strtod((char*) strtok(NULL, " \n"), (char**) NULL) * 1e6;
      elg_cycles_per_sec = (elg_ui8) hz;
    }
    if (!strncmp("flags", line, 5))
    { 
      strtok(line, ":");
      while ( (token = (char*) strtok(NULL, " \n"))!=NULL)
	if (strcmp(token,"tsc")==0)
	{
	  elg_cpu_has_tsc=1;
	  break;
	}
      break;
    }
  }
  
  fclose(cpuinfofp);
}

void elg_pform_mpi_init()
{
}

void elg_pform_mpi_finalize()
{
} 

char* elg_pform_gdir()
{
  return ".";
}

char* elg_pform_ldir()
{
  #ifdef PFORM_LDIR
    return PFORM_LDIR;
  #else
    return "/tmp";
  #endif
}


int elg_pform_is_gclock()
{
  return 0;
}

double elg_pform_wtime()
{
  elg_ui8 clock_value;
  
  /* fast assembler code to access the cycle counter (TSC) */
  asm volatile ("rdtsc" : "=a"(*(elg_ui4 *)&clock_value),
		"=d"(*(((elg_ui4 *)&clock_value)+1)));  
  
  return (double) (clock_value) / (double) elg_cycles_per_sec;
}

/* platform name */
char*  elg_pform_name()
{
  return "Linux Cluster";
}

int  elg_pform_num_nodes()
{
  return 1;
}

long elg_pform_node_id()
{
  return gethostid(); 
}

char* elg_pform_node_name()
{
  static char host_name[20];

  /* get host name */
  gethostname(host_name, 20);
  return host_name;
}

int elg_pform_num_cpus()
{
   return elg_cpu_count;
}
