/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (C) 1998-2003                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "elg_omplock.h"
#include "elg_ompreg.h"
#include "elg_trc.h"

#include "elg_pomp2.h"
#include "elg_pomp2_ctc.h"
#include "elg_pomp2_grp.h"

#define GUARDED_ENTER(id, G, L) \
  if ( pomp_active && pomp_grp_level[G] >= L ) { \
    struct ElgRegDescr* data = (struct ElgRegDescr*)(*handle); \
    elg_enter(data->id); \
  }

#define CTC_GUARDED_ENTER(id, G, L) \
  if ( pomp_active && pomp_grp_level[G] >= L ) { \
    struct ElgRegDescr* data; \
    if ( *handle == 0 ) POMP_Save_get_handle(handle, ctc); \
    d = (struct ElgRegDescr*)(*handle); \
    elg_enter(data->id); \
  }

#define GUARDED_EXIT(G, L) \
  if ( pomp_active && pomp_grp_level[G] >= L ) { \
    elg_exit(); \
  }

#define GUARDED_COLL_EXIT(G, L) \
  if ( pomp_active && pomp_grp_level[G] >= L ) { \
    elg_omp_collexit(); \
  }

struct ElgRegDescr {
  elg_ui4 rid;     /* region id */
  elg_ui4 lockid;  /* lockid for critical */
  elg_ui4 sbrid;   /* region id of enclosed structured block */
};

static int pomp_active = 0;

/*
 * Control Routines
 */
void POMP_Finalize(void) {
  static int pomp_finalize_called = 0;

  if ( ! pomp_finalize_called ) {
    pomp_finalize_called = 1;
    elg_close();
  }
}

p_int32 POMP_Init(void) {
  static int pomp_init_called = 0;

  if ( ! pomp_init_called ) {
    pomp_init_called = 1;
    elg_open();

    atexit(POMP_Finalize);
    pomp_init_groups();
    elg_omp_register();
    pomp_active = 1;
  }
  return 0;
}

void POMP_On(void) {
  pomp_active = 1;
}

void POMP_Off(void) {
  pomp_active = 0;
}

void POMP_Set_group(enum pomp_group_t group, enum pomp_grpval_t level) {
  pomp_grp_level[group] = level;
}

p_int32 POMP_Get_handle(POMP_Handle_t* handle, char ctc[]) {
  ctc_field f;
  elg_ui1 rtype;
  char* rname = "?";
  char* dname = "?";
  char* fname = 0;
  char* ptr = 0;
  const char* rdesc = "OMP";
  struct ElgRegDescr* data = malloc(sizeof(struct ElgRegDescr));
  elg_ui4 fid  = ELG_NO_ID;
  elg_ui4 begln = ELG_NO_LNO;
  elg_ui4 endln = ELG_NO_LNO;
  elg_ui4 begln2 = ELG_NO_LNO;
  elg_ui4 endln2 = ELG_NO_LNO;

  data->rid  = ELG_NO_ID;
  data->lockid = ELG_NO_ID;
  data->sbrid = ELG_NO_ID;

  ctc_init(&f, ctc);
  while ( ctc_get_next_field(&f) ) {
    enum ctc_field_t type = ctc_field_type(f.attr);
    switch ( type ) {
      case CTC_RTYPE:
	conv_ctc_region_type(f.val, &rtype, &rname);
        break;
      case CTC_NAME:
	dname = f.val;
        break;
      case CTC_SSCL:
	fname = f.val;
	if ((ptr = strchr(fname, ':')) != 0 ) {
	  *ptr = '\0';
	  ++ptr;
	  begln = strtol(ptr, &ptr, 10);
	  if ( *ptr == ':' ) {
	    ++ptr;
	    begln2 = strtol(ptr, &ptr, 10);
	  }
	}
	fid = elg_def_file(fname);
        break;
      case CTC_ESCL:
	fname = f.val;
	if ((ptr = strchr(fname, ':')) != 0 ) {
	  *ptr = '\0';
	  ++ptr;
	  endln = strtol(ptr, &ptr, 10);
	  if ( *ptr == ':' ) {
	    ++ptr;
	    endln2 = strtol(ptr, &ptr, 10);
	  }
	}
	/*fid = elg_def_file(fname); ???*/
        break;
    }
  }
  switch ( rtype ) {
    case ELG_USER_REGION:
    case ELG_FUNCTION:
      rdesc = "USR";
      rname = dname;
      break;
    case ELG_OMP_CRITICAL:
      data->sbrid = elg_def_region("!$omp critical sblock", fid,
				   (begln2 == ELG_NO_LNO ? begln2 : begln2+1),
				   (endln == ELG_NO_LNO ? endln : endln-1),
                                   "OMP", ELG_OMP_CRITICAL_SBLOCK);
      if ( dname[0] != '?' ) {
	char *n = malloc(24 + strlen(rname));
        sprintf(n, "!$omp critical %s", rname);
        data->lockid = elg_lock_init(rname);
	rname = n;
      } else {
        data->lockid = elg_lock_init("default");
      }
      break;
    case ELG_OMP_SECTIONS:
      data->sbrid = elg_def_region("!$omp section", fid,
				   (begln2 == ELG_NO_LNO ? begln2 : begln2+1),
				   (endln == ELG_NO_LNO ? endln : endln-1),
                                   "OMP", ELG_OMP_SECTION);
      break;
    case ELG_OMP_SINGLE:
      data->sbrid = elg_def_region("!$omp single sblock", fid,
				   (begln2 == ELG_NO_LNO ? begln2 : begln2+1),
				   (endln == ELG_NO_LNO ? endln : endln-1),
                                   "OMP", ELG_OMP_SINGLE_SBLOCK);
      break;

  }
  data->rid = elg_def_region(rname, fid, begln, endln, rdesc, rtype);
  *handle = data;
  return 0;
}

p_int32 POMP_Save_get_handle(POMP_Handle_t* handle, char ctc[]) {
  p_int32 r;

  if (omp_in_parallel()) {
    #pragma omp critical(pomp_get_handle)
    {
      if ( *handle == 0 ) r = POMP_Get_handle(handle, ctc);
    }
  } else {
    r = POMP_Get_handle(handle, ctc);
  }
  return r;
}

/*
 * Event Routines Group PARALLEL
 */
p_int32 POMP_Parallel_enter(POMP_Handle_t* handle, p_int32 thread_id,
                          p_int32 num_threads, p_int32 if_expr_result,
                          char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_PARALLEL] >= GV_ENTEREXIT ) {
    elg_omp_fork();
  }
}

p_int32 POMP_Parallel_begin(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_ENTER(rid, GRP_PARALLEL, GV_BEGINEND);
}

p_int32 POMP_Parallel_end(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_PARALLEL, GV_BEGINEND);
}

p_int32 POMP_Parallel_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  if ( pomp_active && pomp_grp_level[GRP_PARALLEL] >= GV_ENTEREXIT ) {
    elg_omp_join();
  }
}


/*
 * Event Routines Group LOOP
 */
p_int32 POMP_Loop_enter(POMP_Handle_t* handle, p_int32 thread_id,
                      p_int64 chunk_size, p_int64 init_iter, p_int64 final_iter,
                      p_int64 incr, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_LOOP, GV_ENTEREXIT);
}

/* -- NOT USED
p_int32 POMP_Loop_chunk_begin(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 init_iter, p_int64 final_iter) {}
p_int32 POMP_Loop_iter_begin(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 iter) {}
p_int32 POMP_Loop_iter_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 iter) {}
p_int32 POMP_Loop_iter_end(POMP_Handle_t* handle, p_int32 thread_id) {}
p_int32 POMP_Loop_chunk_end(POMP_Handle_t* handle, p_int32 thread_id) {}
-- */

p_int32 POMP_Loop_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_LOOP, GV_ENTEREXIT);
}


/*
 * Event Routines Group WORKSHARE
 */
p_int32 POMP_Workshare_enter(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_WSHARE, GV_ENTEREXIT);
}

/* -- NOT USED
p_int32 POMP_Workshare_begin(POMP_Handle_t* handle, p_int32 thread_id) {}
p_int32 POMP_Workshare_end(POMP_Handle_t* handle, p_int32 thread_id) {}
-- */

p_int32 POMP_Workshare_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_WSHARE, GV_ENTEREXIT);
}

p_int32 POMP_Sections_enter(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_WSHARE, GV_ENTEREXIT);
}

p_int32 POMP_Section_begin(POMP_Handle_t* handle, p_int32 thread_id,
                         p_int32 section_num) {
  GUARDED_ENTER(sbrid, GRP_WSHARE, GV_BEGINEND);
}

p_int32 POMP_Section_end(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_WSHARE, GV_BEGINEND);
}

p_int32 POMP_Sections_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_WSHARE, GV_ENTEREXIT);
}

p_int32 POMP_Single_enter(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_WSHARE, GV_ENTEREXIT);
}

p_int32 POMP_Single_begin(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_ENTER(sbrid, GRP_WSHARE, GV_BEGINEND);
}

p_int32 POMP_Single_end(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_WSHARE, GV_BEGINEND);
}

p_int32 POMP_Single_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_WSHARE, GV_ENTEREXIT);
}


/*
 * Event Routines Group SYNC
 */
p_int32 POMP_Critical_enter(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_SYNC, GV_ENTEREXIT);
}

p_int32 POMP_Critical_begin(POMP_Handle_t* handle, p_int32 thread_id) {
  if ( pomp_active && pomp_grp_level[GRP_SYNC] >= GV_BEGINEND ) {
    struct ElgRegDescr* data = (struct ElgRegDescr*)(*handle);
    elg_omp_alock(data->lockid);
    elg_enter(data->sbrid);
  }
}

p_int32 POMP_Critical_end(POMP_Handle_t* handle, p_int32 thread_id) {
  if ( pomp_active && pomp_grp_level[GRP_SYNC] >= GV_BEGINEND ) {
    struct ElgRegDescr* data = (struct ElgRegDescr*)(*handle);
    elg_exit();
    elg_omp_rlock(data->lockid);
  }
}

p_int32 POMP_Critical_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_SYNC, GV_ENTEREXIT);
}

p_int32 POMP_Master_begin(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_SYNC, GV_BEGINEND);
}

p_int32 POMP_Master_end(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_SYNC, GV_BEGINEND);
}

p_int32 POMP_Barrier_enter(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_SYNC, GV_ENTEREXIT);
}

p_int32 POMP_Barrier_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_SYNC, GV_ENTEREXIT);
}

p_int32 POMP_Implicit_barrier_enter(POMP_Handle_t* handle, p_int32 thread_id,
                                  char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_SYNC, GV_ENTEREXIT);
}

p_int32 POMP_Implicit_barrier_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_COLL_EXIT(GRP_SYNC, GV_ENTEREXIT);
}

p_int32 POMP_Flush_event(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_SYNC, GV_EVENT);
  GUARDED_EXIT(GRP_SYNC, GV_EVENT);
}

p_int32 POMP_Atomic_event(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_SYNC, GV_EVENT);
  GUARDED_EXIT(GRP_SYNC, GV_EVENT);
}

/* -- NOT USED
p_int32 POMP_Ordered_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]) {}
p_int32 POMP_Ordered_begin(POMP_Handle_t* handle, p_int32 thread_id) {}
p_int32 POMP_Ordered_end(POMP_Handle_t* handle, p_int32 thread_id) {}
p_int32 POMP_Ordered_exit(POMP_Handle_t* handle, p_int32 thread_id) {}
-- */


/*
 * Event Routines Group USER
 */
p_int32 POMP_Function_enter(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_USER, GV_ENTEREXIT);
}

p_int32 POMP_Function_begin(POMP_Handle_t* handle, p_int32 thread_id, char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_USER, GV_BEGINEND);
}

p_int32 POMP_Function_end(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_USER, GV_BEGINEND);
}

p_int32 POMP_Function_exit(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_USER, GV_ENTEREXIT);
}

/* -- NOT USED
p_int32 POMP_Function_event(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]) {}
-- */

p_int32 POMP_User_region_begin(POMP_Handle_t* handle, p_int32 thread_id,
                             char ctc[]) {
  /*CTC_*/GUARDED_ENTER(rid, GRP_USER, GV_BEGINEND);
}

p_int32 POMP_User_region_end(POMP_Handle_t* handle, p_int32 thread_id) {
  GUARDED_EXIT(GRP_USER, GV_BEGINEND);
}

/* -- NOT USED
p_int32 POMP_User_event(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]) {}
-- */


/*
 * Event Routines Group RUNTIME
 */
p_int32 POMP_Init_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                           omp_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_INIT_LOCK]);
    elg_lock_init(lock_id);
    elg_exit();
  } else {
    elg_lock_init(lock_id);
  }
}

p_int32 POMP_Destroy_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                              omp_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_DESTROY_LOCK]);
    elg_lock_destroy(lock_id);
    elg_exit();
  } else {
    elg_lock_destroy(lock_id);
  }
}

p_int32 POMP_Set_lock_enter(POMP_Handle_t* handle, p_int32 thread_id,
                          omp_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_ENTEREXIT ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_LOCK]);
  }
}

p_int32 POMP_Set_lock_exit(POMP_Handle_t* handle, p_int32 thread_id,
                         omp_lock_t* lock_id) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_ENTEREXIT ) {
    elg_omp_alock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Set_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                          omp_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_LOCK]);
    elg_omp_alock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Unset_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                            omp_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_UNSET_LOCK]);
    elg_omp_rlock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Test_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                           omp_lock_t* lock_id, p_int32 was_lock_set, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_TEST_LOCK]);
    if (was_lock_set) elg_omp_alock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Init_nest_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                                omp_nest_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_INIT_NEST_LOCK]);
    elg_lock_init(lock_id);
    elg_exit();
  } else {
    elg_lock_init(lock_id);
  }
}

p_int32 POMP_Destroy_nest_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                                   omp_nest_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_DESTROY_NEST_LOCK]);
    elg_lock_destroy(lock_id);
    elg_exit();
  } else {
    elg_lock_destroy(lock_id);
  }
}

p_int32 POMP_Set_nest_lock_enter(POMP_Handle_t* handle, p_int32 thread_id,
                               omp_nest_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_ENTEREXIT ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_NEST_LOCK]);
  }
}

p_int32 POMP_Set_nest_lock_exit(POMP_Handle_t* handle, p_int32 thread_id,
                              omp_nest_lock_t* lock_id) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_ENTEREXIT ) {
    elg_omp_alock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Set_nest_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                               omp_nest_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_NEST_LOCK]);
    elg_omp_alock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Unset_nest_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                                 omp_nest_lock_t* lock_id, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_UNSET_NEST_LOCK]);
    elg_omp_rlock(elg_lock_id(lock_id));
    elg_exit();
  }
}

p_int32 POMP_Test_nest_lock_event(POMP_Handle_t* handle, p_int32 thread_id,
                                omp_nest_lock_t* lock_id,
                                p_int32 was_lock_set, char ctc[]) {
  if ( pomp_active && pomp_grp_level[GRP_RUNTIME] >= GV_EVENT ) {
    elg_enter(elg_omp_regid[ELG__OMP_TEST_NEST_LOCK]);
    if (was_lock_set) elg_omp_alock(elg_lock_id(lock_id));
    elg_exit();
  }
}

/* -- NOT USED
p_int32 POMP_Set_num_threads_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_threads, char ctc[]) {} 
p_int32 POMP_Get_num_threads_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_threads, char ctc[]) {} 
p_int32 POMP_Get_max_threads_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_threads, char ctc[]) {}
p_int32 POMP_Get_thread_num_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 thread_num, char ctc[]) {} 
p_int32 POMP_Get_num_procs_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_procs, char ctc[]) {} 
p_int32 POMP_In_parallel_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_parallel, char ctc[]) {} 
p_int32 POMP_Set_dynamic_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_dynamic, char ctc[]) {} 
p_int32 POMP_Get_dynamic_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_dynamic, char ctc[]) {}
p_int32 POMP_Set_nested_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_nested, char ctc[]) {} 
p_int32 POMP_Get_nested_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_nested, char ctc[]) {}
-- */
