/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdlib.h>
#include "elg_error.h"
#include "elg_readcb.h"

#include <stdio.h>

void elg_readcb_ALL( elg_ui1 type, 
		     elg_ui1 length,
                     void* userdata )
{
  printf("(%3u,%3u): ", type, length); 
}

void elg_readcb_STRING( elg_ui4 strid,
		      elg_ui1 cntc,
		      const char* str,
                      void* userdata ) 
{
  printf("%-16s", "<STRING>");    
  printf("  strid  : %3u,", strid);
  printf("  cntc   : %3u,", cntc);
  printf("  str    : %.250s ", str); 
  printf("\n");
}

void elg_readcb_STRING_CNT( const char* str, void* userdata ) 
{
  printf("%-16s", "<STRING_CNT>");
  printf("  str    : %.255s ", str);   
  printf("\n");
}

void elg_readcb_MACHINE( elg_ui4 mid,
		       elg_ui4 nodec,
		       elg_ui4 mnid,
                       void* userdata ) 
{
  printf("%-16s", "<MACHINE>");
  printf("  mid    : %3u,", mid);
  printf("  nodec  : %3u,", nodec);
  printf("  mnid   : %3u", mnid);
  printf("\n");
}

void elg_readcb_NODE( elg_ui4 nid,
		    elg_ui4 mid,
		    elg_ui4 cpuc,
		    elg_ui4 nnid, 
		    elg_d8  cr,
                    void* userdata ) 
{
  printf("%-16s", "<NODE>");
  printf("  nid    : %3u,", nid);
  printf("  mid    : %3u,", mid);
  printf("  cpuc   : %3u,", cpuc);
  printf("  nnid   : %3u,", nnid);
  printf("  cr     : %10.6f", cr);
  printf("\n");
}

void elg_readcb_PROCESS( elg_ui4 pid,
		       elg_ui4 pnid,
                       void* userdata ) 
{
  printf("%-16s", "<PROCESS>");
  printf("  pid    : %3u,", pid);
  printf("  pnid   : %3u", pnid);
  printf("\n");
}

void elg_readcb_THREAD( elg_ui4 tid,
		      elg_ui4 pid,
		      elg_ui4 tnid,
                      void* userdata ) 
{
  printf("%-16s", "<THREAD>");
  printf("  tid    : %3u,", tid);
  printf("  pid    : %3u,", pid);
  printf("  tnid   : %3u", tnid);
  printf("\n");
}

void elg_readcb_LOCATION( elg_ui4 lid,
			elg_ui4 mid,
			elg_ui4 nid,
			elg_ui4 pid,
			elg_ui4 tid,
                        void* userdata ) 
{
  printf("%-16s", "<LOCATION>");
  printf("  lid    : %3u,", lid);
  printf("  mid    : %3u,", mid);
  printf("  nid    : %3u,", nid);
  printf("  pid    : %3u,", pid);
  printf("  tid    : %3u", tid);
  printf("\n");
}

void elg_readcb_FILE( elg_ui4 fid,
		    elg_ui4 fnid,
                    void* userdata ) 
{
  printf("%-16s", "<FILE>");
  printf("  fid    : %3u,", fid);
  printf("  fnid   : %3u", fnid);
  printf("\n");  
}

void elg_readcb_REGION( elg_ui4 rid,
		      elg_ui4 rnid,
		      elg_ui4 fid,
		      elg_ui4 begln,
		      elg_ui4 endln,
		      elg_ui4 rdid,
		      elg_ui1 rtype,
                      void* userdata ) 
{
  printf("%-16s", "<REGION>");
  printf("  rid    : %3u,", rid);
  printf("  rnid   : %3u,", rnid);
  printf("  fid    : %3u,", fid);
  printf("  begln  : %3u,", begln);
  printf("  endln  : %3u,", endln);
  printf("  rdid   : %3u,", rdid);
  printf("  rtype  : %3u", rtype);
  printf("\n");
}

void elg_readcb_CALL_SITE( elg_ui4 csid,
			 elg_ui4 fid,
			 elg_ui4 lno,
			 elg_ui4 erid,
			 elg_ui4 lrid,
                         void* userdata ) 
{
  printf("%-16s", "<CALL_SITE>");
  printf("  csid   : %3u,", csid);
  printf("  fid    : %3u,", fid);
  printf("  lno    : %3u,", lno);
  printf("  erid   : %3u,", erid);
  printf("  lrid   : %3u", lrid);
  printf("\n");
}

void elg_readcb_METRIC( elg_ui4 metid,
		      elg_ui4 metnid,
		      elg_ui4 metdid,
		      elg_ui1 metdtype,
		      elg_ui1 metmode,
		      elg_ui1 metiv,
                      void* userdata ) 
{
  printf("%-16s", "<METRIC>");
  printf("  metid  : %3u,", metid);
  printf("  metnid : %3u,", metnid);
  printf("  metdid : %3u,", metdid);
  printf("  metdtype : %3u,", metdtype);
  printf("  metmode : %3u,", metmode);
  printf("  metiv  : %3u", metiv);
  printf("\n");
}

void elg_readcb_MPI_COMM( elg_ui4 cid,
			elg_ui4 grpc,
			elg_ui1 grpv[],
                        void* userdata ) 
{
  int i, j;

  printf("%-16s", "<MPI_COMM>");
  printf("  cid    : %3u,", cid);	  
  printf("  grpc   : %3u, group  : ( ", grpc);	  
  
  for (i = 0; i < grpc; i++)
    for (j = 0; j < 8; j++)
      printf("%u", (grpv[i] & (1 << (j % 8))  ? 1 : 0 ));
  
  printf(" )");
  printf("\n");
}

void elg_readcb_CART_TOPOLOGY( elg_ui4 topid,
			       elg_ui1 ndims,
			       elg_ui4 dimv[], 
			       elg_ui1 periodv[],
			       void* userdata ) 
{
  int i;

  printf("%-16s", "<CART_TOPOLOGY>");
  printf("  topid  : %3u,", topid);	  
  printf("  ndims  : %3u,", ndims);	  
  printf("  dimv   : ( "); 
  for ( i = 0 ; i < ndims-1 ; i++ ) 
    printf("%u, ", dimv[i]);
  printf("%u", dimv[ndims-1]);
  printf(" ),");

  printf(" periodv : ( ");
  for ( i = 0 ; i < ndims-1 ; i++ ) 
    printf("%u, ", periodv[i]);
  printf("%u", periodv[ndims-1]);
  printf(" )");

  printf("\n");
}

void elg_readcb_CART_COORDS( elg_ui4 topid,
			     elg_ui4 lid,
			     elg_ui1 ndims,
			     elg_ui4 coordv[],
			     void* userdata)
{
  int i;

  printf("%-16s", "<CART_COORDS>");
  printf("  topid  : %3u,", topid);	  
  printf("  lid    : %3u,", lid);	  
  printf("  ndims  : %3u,", ndims);	 

  printf(" coordv  :  ( ");
  for ( i = 0 ; i < ndims-1 ; i++ ) 
    printf("%u, ", coordv[i]);
  printf("%u", coordv[ndims-1]);
  printf(" )");

  printf("\n");
}

void elg_readcb_OFFSET( elg_d8  ltime,
		      elg_d8  offset,
                      void* userdata ) 
{
  printf("%-16s", "<OFFSET>");
  printf("  ltime  : %10.6f,", ltime);
  printf("  offset : %10.6f", offset);
  printf("\n");
}

void elg_readcb_NUM_EVENTS( elg_ui4 eventc, void* userdata ) 
{
  printf("%-16s", "<NUM_EVENTS>");
  printf("  eventc : %3u", eventc);
  printf("\n");
}

void elg_readcb_LAST_DEF( void* userdata ) 
{
  printf("%-16s", "<LAST_DEF>");
  printf("\n");
}


void elg_readcb_ENTER( elg_ui4 lid, 
		     elg_d8  time,
		     elg_ui4 rid,
		     elg_ui1 metc,
		     elg_ui8 metv[],
                     void* userdata ) 
{
  int i;

  printf("%-16s", "<ENTER>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  rid    : %3u,", rid);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_ENTER_CS( elg_ui4 lid, 
			elg_d8  time,
			elg_ui4 csid,
			elg_ui1 metc,
			elg_ui8 metv[],
                        void* userdata ) 
{
  int i;

  printf("%-16s", "<ENTER>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  csid   : %3u,", csid);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_EXIT( elg_ui4 lid, 
		    elg_d8  time,
		    elg_ui1 metc,
		    elg_ui8 metv[],
                    void* userdata ) 
{
  int i;

  printf("%-16s", "<EXIT>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_MPI_SEND( elg_ui4 lid, 
			elg_d8  time,
			elg_ui4 dlid,
			elg_ui4 cid,
			elg_ui4 tag,      
			elg_ui4 sent,
                        void* userdata ) 
{
  printf("%-16s", "<MPI_SEND>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  dlid    : %3u,", dlid);
  printf("  cid    : %3u,", cid);
  printf("  tag    : %3u,", tag);
  printf("  sent   : %3u", sent);
  printf("\n");
}

void elg_readcb_MPI_RECV( elg_ui4 lid, 
			elg_d8  time,
			elg_ui4 slid,
			elg_ui4 cid,
			elg_ui4 tag,
                        void* userdata ) 
{
  printf("%-16s", "<MPI_RECV>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  slid    : %3u,", slid);
  printf("  cid    : %3u,", cid);
  printf("  tag    : %3u,", tag);
  printf("\n");
}

void elg_readcb_MPI_COLLEXIT( elg_ui4 lid, 
			    elg_d8  time,
			    elg_ui1 metc,
			    elg_ui8 metv[], 
			    elg_ui4 rlid,
			    elg_ui4 cid,
			    elg_ui4 sent,  
			    elg_ui4 recvd,
                            void* userdata ) 
{
  int i;

  printf("%-16s", "<MPI_COLLEXIT>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("  rlid   : %3u,", rlid);
  printf("  cid    : %3u,", cid);
  printf("  sent   : %3u,", sent);
  printf("  recvd  : %3u", recvd);
  printf("\n");
}

void elg_readcb_OMP_FORK( elg_ui4 lid, 
			elg_d8  time,
                        void* userdata ) 
{
  printf("%-16s", "<OMP_FORK>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f", time);	    
  printf("\n");
}

void elg_readcb_OMP_JOIN( elg_ui4 lid, 
			elg_d8  time,
                        void* userdata ) 
{
  printf("%-16s", "<OMP_JOIN>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f", time);	    
  printf("\n");
}

void elg_readcb_OMP_ALOCK( elg_ui4 lid, 
			 elg_d8  time,
			 elg_ui4 lkid,
                         void* userdata ) 
{
  printf("%-16s", "<OMP_ALOCK>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  lkid   : %3u", lkid);
  printf("\n");
}

void elg_readcb_OMP_RLOCK( elg_ui4 lid, 
			 elg_d8  time,
			 elg_ui4 lkid,
                         void* userdata ) 
{
  printf("%-16s", "<OMP_RLOCK>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  lkid   : %3u", lkid);
  printf("\n");
}

void elg_readcb_OMP_COLLEXIT( elg_ui4 lid, 
			    elg_d8  time,
			    elg_ui1 metc,
			    elg_ui8 metv[],
                            void* userdata ) 
{
  int i;

  printf("%-16s", "<OMP_COLLEXIT>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f,", time);
  printf("  metv    : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_ENTER_DUMP( elg_ui4 lid, 
			  elg_d8  time,
			  elg_ui1 metc,
			  elg_ui8 metv[],
                          void* userdata ) 
{
  int i;

  printf("%-16s", "<ENTER_DUMP>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f", time);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_EXIT_DUMP( elg_ui4 lid, 
			 elg_d8  time,
			 elg_ui1 metc,
			 elg_ui8 metv[],
                         void* userdata ) 
{
  int i;

  printf("%-16s", "<EXIT_DUMP>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f", time);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_LOG_OFF( elg_ui4 lid, 
		       elg_d8  time,
		       elg_ui1 metc,
		       elg_ui8 metv[],
                       void* userdata ) 
{
  int i;

  printf("%-16s", "<LOG_OFF>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f", time);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

void elg_readcb_LOG_ON( elg_ui4 lid, 
		      elg_d8  time,
		      elg_ui1 metc,
		      elg_ui8 metv[],
                      void* userdata ) 
{
  int i;

  printf("%-16s", "<LOG_ON>");
  printf("  lid    : %3u,", lid);
  printf("  time   : %10.6f", time);
  printf("  metv   : ( ");
  for (i = 0; i < metc; i++)
    printf("%3lld ", metv[i]);
  printf(")");
  printf("\n");
}

int main(int argc, char **argv) 
{
  ElgRCB* handle;

  if (argc != 2)
    elg_error_msg("Bad arguments\nUsage: elg_print <file>");
    
  handle = elg_read_open(argv[1]);
  if (handle == NULL)
    exit(EXIT_FAILURE);

  while ( elg_read_next(handle, NULL) );

  elg_read_close(handle);
    
  return 0;
}



















