/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (C) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#if OPENMP
#  include <omp.h>
#endif
#include <mpi.h>

double do_work(int units) {
int i, j;
double x1, x2;

  for (i=0; i<units; ++i) {
    for (j=0; j<10000; ++j) {
      x1 = j*3.1415;
      x2 = x1 * 42.0 * (x1 - 42.0);
    }
  }
  return x2;
}

void step(int id, int tid, int i) {
  /* printf("%d:%d: step %d\n", id, tid, i); */
  do_work(100);
}

void sequential(int id) {
  int i, tid=0;

  do_work(100);
  #pragma omp parallel private(tid)
  {
    #ifdef OPENMP
    tid = omp_get_thread_num();
    #endif
    #pragma omp for
    for (i=0; i<12; i++) {
      step(id, tid, i);
    }
  }
  do_work(100);
}

void parallel(MPI_Comm comm) {
  int id=0, numprocs, value=42;

  MPI_Comm_size(comm, &numprocs);
  MPI_Comm_rank(comm, &id);
  MPI_Bcast(&value, 1, MPI_INT, numprocs-1, comm);

  sequential(id);
  MPI_Barrier(comm);
  sequential(id);
  MPI_Barrier(comm);
  sequential(id);
  MPI_Barrier(comm);
}

int main(int argc, char* argv[]) {
  int id=0, numprocs;
  int color;
  MPI_Comm local;

  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD, &id);
  color = (id >= numprocs/2);
  MPI_Comm_split(MPI_COMM_WORLD, color, id, &local);

  parallel(MPI_COMM_WORLD);
  parallel(local);
  parallel(MPI_COMM_WORLD);

  MPI_Comm_free(&local);
  MPI_Finalize();

  return 0;
}
