#!/usr/bin/python

import sys

from earl import *
from string import *

from CallTree import *
from PatternTree import *
from Profile import *
from Report import *

class Analyzer:

    def __init__(self, trace):

        self._trace = trace

        # set up callbacks

        self._callbacks = {}

        # primitive events
        self._callbacks['ENTER']       = []
        self._callbacks['EXIT']        = []
        self._callbacks['MPICEXIT']    = []
        self._callbacks['OMPCEXIT']    = []
        self._callbacks['SEND']        = []
        self._callbacks['RECV']        = []
        self._callbacks['FORK']        = []
        self._callbacks['JOIN']        = []
        self._callbacks['ALOCK']       = []
        self._callbacks['RLOCK']       = []

        # compound events
        self._callbacks['RECV_OP']      = []
        self._callbacks['LATE_SEND']   = []
        self._callbacks['LATE_RECV']   = []
        self._callbacks['MPI_COLL']    = []
        self._callbacks['LOCK_COMP']   = []
        self._callbacks['OMP_COLL']    = []
        self._callbacks['TEAM_CREATE'] = []

        # create profile
        self._profile = Profile(trace, self)
        # create pattern tree
        self._ptree = PatternTree(trace, self)

    def subscribe(self, event_name, callback):

        self._callbacks[event_name].append(callback)
        if event_name == 'EXIT':
            self._callbacks['MPICEXIT'].append(callback)
            self._callbacks['OMPCEXIT'].append(callback)


    def run(self):

        pos = 1
        percent = 0
        while pos <= self._trace.get_nevents():
            self._step(pos)
            new = int(round(100 * (float(pos) / self._trace.get_nevents()), 0))
            if new > percent:
                percent = new
                print >>sys.stderr, '\r', percent, '%', 
            pos += 1
        print

        # create call tree
        self._ctree = CallTree(self._trace)

        # check for missing EXIT events (i.e., whether stack empty)
        last_pos = self._trace.get_nevents();
        for loc_id in range(0, self._trace.get_nlocs()):
            if len(self._trace.stack(last_pos, loc_id)) != 0:
                print "EXPERT: Error in analysis. Trace file incomplete because of missing EXIT events." 
                sys.exit(1);
       
        # callbacks for profile
        self._profile.exclusify(self._ctree)
        for key in self._ptree.keys():
            self._ptree.data(key).end(self._profile, self._ctree)

        # generate report
        cube_file   = self._trace.get_name()[0:-3] + "cube"
        cube = Report(self._ptree, self._ctree, self._trace)
        cube.write(cube_file)

        return
        

    def _step(self, pos):

        curr = self._trace.event(pos)        
        ce_list = []
        
        # callbacks for primitive events
        for cb in self._callbacks[curr['type']]:
            ce = cb(curr)
            if ce != None:
                ce_list.append(ce)
                
        # callbacks for compound events
        while len(ce_list) > 0:
            ce = ce_list.pop()
            for cb in self._callbacks[ce['TYPE']]:
                new_ce = cb(ce)
                if new_ce != None:
                    ce_list.append(new_ce)



