from cube import *
from earl import *

from CallTree import *
from PatternTree import *


class Report:

    def __init__(self, ptree, ctree, trace):
        self._ptree = ptree
        self._ctree = ctree
        self._trace = trace

        # create CUBE data structure
        self._cube = Cube()
        
        # create id mapping tables
        self._pat_map   = {}
        self._module_map = {}
        self._region_map = {}
        self._csite_map  = {}
        self._cnode_map  = {}
        self._mach_map   = {}
        self._node_map   = {}
        self._proc_map   = {}
        self._thrd_map   = {}
        self._loc_map    = {}

        # absent information
        self._pat_map[None] = -1
        self._module_map["UNKNOWN"] = self._cube.def_module("UNKNOWN")
        self._region_map[-1] = self._cube.def_region("UNKNOWN", -1, -1, "PSEUDO", self._module_map["UNKNOWN"])
        self._cnode_map[None] = -1

        # define metric tree
        self._def_cube_met()
        
        # define call tree
        self._def_cube_module()
        self._def_cube_region()
        self._def_cube_csite()
        self._def_cube_cnode()

        # define location tree
        self._def_cube_machine()        
        self._def_cube_proc()

        # fill in data
        self._set_cube_sev()

    def _def_cube_met(self):
        for pat_id in self._ptree.dfkeys():
            pat = self._ptree.data(pat_id)
            self._pat_map[pat.__class__.__name__] = self._cube.def_met(pat.__doc__, "sec", pat.descr(),
                                                                       self._pat_map[pat.parent()]);
       
    def _def_cube_module(self):
        for file_id in range(0, self._trace.get_nfiles()):
            file = self._trace.get_file(file_id)
            path = "path"
            self._module_map[file] = self._cube.def_module(file)
        
    def _def_cube_region(self):
        for reg_id in range(0, self._trace.get_nregs()):
            reg = self._trace.get_reg(reg_id)
            self._region_map[reg_id] = self._cube.def_region(reg['name'], reg['begln'], reg['endln'], 
                                                       reg['descr'], self._module_map[reg['file']])

    def _def_cube_csite(self):
        for csite_id in range(0, self._trace.get_ncsites()):
            csite = self._trace.get_csite(csite_id)
            self._csite_map[csite_id] = self._cube.def_csite(self._module_map[csite['file']],
                                                             csite['line'], self._region_map[csite['callee_id']])
        
    def _def_cube_cnode(self):
        for cnode_id in self._ctree.dfkeys():
            cnode = self._ctree.data(cnode_id)
            if cnode['csite_id'] == -1:
                cube_csite_id = self._cube.def_csite(self._module_map["UNKNOWN"], -1, self._region_map[cnode['callee_id']])
            else:
                cube_csite_id = self._csite_map[cnode['csite_id']]
            self._cnode_map[cnode_id] = self._cube.def_cnode(cube_csite_id,
                                                             self._cnode_map[self._ctree.parent(cnode_id)])
            
    def _def_cube_machine(self):
        grid_id = self._cube.def_grid("Grid")
        for mach_id in range(0, self._trace.get_nmachs()):
            mach = self._trace.get_mach(mach_id)
            self._mach_map[mach_id] = self._cube.def_mach(mach['name'], grid_id)
            self._def_cube_node(mach)
        
    def _def_cube_node(self, mach):
        for node_id in range(0, mach['nnodes']):
            node = self._trace.get_node(mach['id'], node_id)
            self._node_map[(mach['id'], node_id)] = self._cube.def_node(node['name'], self._mach_map[mach['id']])

    def _def_cube_proc(self):
        for proc_id in range(0, self._trace.get_nprocs()):
            proc = self._trace.get_proc(proc_id)
            self._proc_map[proc_id] = self._cube.def_proc(proc['name'], self._node_map[(proc['mach_id'], proc['node_id'])])
            self._def_cube_thrd(proc)
        
    def _def_cube_thrd(self, proc):
        for thrd_id in range(0, proc['nthrds']):
            thrd = self._trace.get_thrd(proc['id'], thrd_id)
            self._thrd_map[(proc['id'], thrd_id)] = self._cube.def_thrd(thrd['name'], self._proc_map[proc['id']])
            self._loc_map[thrd['loc_id']] = self._thrd_map[(proc['id'], thrd_id)]
                     
    def _set_cube_sev(self):
        for pat_id in self._ptree.keys():
            pat = self._ptree.data(pat_id)
            sev = pat.severity()
            for cnode_id in self._ctree.keys():
                for loc_id in range(0, self._trace.get_nlocs()):
                    self._cube.set_sev(self._pat_map[pat_id],
                                       self._cnode_map[cnode_id],
                                       self._loc_map[loc_id],
                                       sev.get(cnode_id, loc_id))

    def write(self, path):
        self._cube.write(path)
    
            
                                                    
